# Code generated for ebcc.

from ebcc import numpy as np
from ebcc.util import pack_2e, einsum, Namespace
from ebcc.precision import types

def energy(f=None, v=None, nocc=None, nvir=None, t1=None, t2=None, t3=None, **kwargs):
    # energy
    x0 = np.zeros((nocc, nocc, nvir, nvir), dtype=types[float])
    x0 += einsum(v.ovov, (0, 1, 2, 3), (0, 2, 3, 1))
    x0 += einsum(v.ovov, (0, 1, 2, 3), (0, 2, 1, 3)) * -0.5
    e_cc = 0
    e_cc += einsum(t2, (0, 1, 2, 3), x0, (0, 1, 3, 2), ()) * 2.0
    x1 = np.zeros((nocc, nvir), dtype=types[float])
    x1 += einsum(f.ov, (0, 1), (0, 1))
    x1 += einsum(t1, (0, 1), x0, (0, 2, 3, 1), (2, 3))
    del x0
    e_cc += einsum(t1, (0, 1), x1, (0, 1), ()) * 2.0
    del x1

    return e_cc

def update_amps(f=None, v=None, space=None, t1=None, t2=None, t3=None, **kwargs):
    nocc = space.ncocc
    nvir = space.ncvir
    naocc = space.naocc
    navir = space.navir
    so = np.ones((nocc,), dtype=bool)
    sv = np.ones((nvir,), dtype=bool)
    sO = space.active[space.correlated][space.occupied[space.correlated]]
    sV = space.active[space.correlated][space.virtual[space.correlated]]

    # T amplitudes
    t1new = np.zeros(((nocc, nvir)), dtype=types[float])
    t1new[np.ix_(so,sv)] += einsum(f.ov, (0, 1), (0, 1))
    t1new[np.ix_(so,sv)] += einsum(f.oo, (0, 1), t1[np.ix_(so,sv)], (1, 2), (0, 2)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(f.vv, (0, 1), t1[np.ix_(so,sv)], (2, 1), (2, 0))
    t1new[np.ix_(so,sv)] += einsum(f.ov, (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 0, 3, 1), (2, 3)) * 2.0
    t1new[np.ix_(so,sv)] += einsum(f.ov, (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 0, 1, 3), (2, 3)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oovv, (2, 0, 3, 1), (2, 3)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovov, (2, 3, 0, 1), (2, 3)) * 2.0
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ooov, (4, 0, 1, 3), (4, 2)) * -1.5
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ooov, (4, 0, 1, 2), (4, 3)) * 0.5
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ooov, (4, 1, 0, 3), (4, 2)) * 0.5
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ooov, (4, 1, 0, 2), (4, 3)) * -0.5
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovvv, (1, 3, 4, 2), (0, 4))
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovvv, (1, 3, 4, 2), (0, 4))
    t1new[np.ix_(so,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovvv, (1, 2, 4, 3), (0, 4)) * -1.0
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 0, 2, 5, 1, 3), (4, 5)) * 0.5
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 0, 2, 5, 3, 1), (4, 5)) * -0.5
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 0, 2, 1, 5, 3), (4, 5)) * -0.25
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 0, 2, 3, 5, 1), (4, 5)) * 0.25
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 2, 0, 5, 3, 1), (4, 5))
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (0, 4, 2, 1, 5, 3), (4, 5)) * 0.25
    t1new[np.ix_(sO,sV)] += einsum(v.OVOV, (0, 1, 2, 3), t3, (0, 4, 2, 3, 5, 1), (4, 5)) * -0.25
    t1new[np.ix_(so,sv)] += einsum(f.ov, (0, 1), t1[np.ix_(so,sv)], (2, 1), t1[np.ix_(so,sv)], (0, 3), (2, 3)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ooov, (4, 0, 2, 1), (4, 3))
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ooov, (4, 2, 0, 1), (4, 3)) * -2.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovvv, (2, 1, 4, 3), (0, 4)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovvv, (2, 3, 4, 1), (0, 4)) * 2.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (2, 1, 3, 5), (0, 4)) * -1.5
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (2, 1, 3, 4), (0, 5)) * 0.5
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (2, 5, 3, 1), (0, 4)) * 0.5
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (2, 4, 3, 1), (0, 5)) * -0.5
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 4, 3, 5), (2, 1)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 5, 3, 4), (2, 1))
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 4, 3, 5), (2, 1)) * -1.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 1, 3, 5), (2, 4)) * 4.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 1, 3, 4), (2, 5)) * -2.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 5, 3, 1), (2, 4)) * -2.0
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovov, (0, 4, 3, 1), (2, 5))
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), v.ovov, (2, 1, 4, 3), (0, 5))
    t1new[np.ix_(so,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), v.ovov, (2, 3, 4, 1), (0, 5)) * -2.0
    t2new = np.zeros(((nocc, nocc, nvir, nvir)), dtype=types[float])
    t2new[np.ix_(so,so,sv,sv)] += einsum(v.ovov, (0, 1, 2, 3), (0, 2, 1, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.oo, (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 1, 3, 4), (0, 2, 4, 3)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.oo, (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 1, 3, 4), (2, 0, 3, 4)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.vv, (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 1, 4), (2, 3, 0, 4))
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.vv, (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 1), (2, 3, 4, 0))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ooov, (2, 0, 3, 4), (2, 3, 1, 4)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ooov, (2, 0, 3, 4), (3, 2, 4, 1)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovvv, (2, 3, 4, 1), (0, 2, 4, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovvv, (2, 3, 4, 1), (2, 0, 3, 4))
    t2new[np.ix_(sO,sO,sV,sV)] += einsum(f.OV, (0, 1), t3, (2, 3, 0, 4, 5, 1), (2, 3, 4, 5))
    t2new[np.ix_(sO,sO,sV,sV)] += einsum(f.OV, (0, 1), t3, (2, 3, 0, 4, 5, 1), (3, 2, 5, 4))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.oooo, (4, 0, 5, 1), (4, 5, 2, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.oovv, (4, 1, 5, 3), (0, 4, 2, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.oovv, (4, 1, 5, 2), (0, 4, 5, 3)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.oovv, (4, 1, 5, 3), (4, 0, 5, 2)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.oovv, (4, 1, 5, 2), (4, 0, 3, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovov, (4, 5, 1, 3), (0, 4, 2, 5)) * 2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovov, (4, 5, 1, 2), (0, 4, 3, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovov, (4, 5, 1, 3), (4, 0, 5, 2)) * 2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.ovov, (4, 5, 1, 2), (4, 0, 5, 3)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), v.vvvv, (4, 2, 5, 3), (0, 1, 4, 5))
    t2new[np.ix_(so,sO,sV,sV)] += einsum(v.oOOV, (0, 1, 2, 3), t3, (4, 1, 2, 5, 6, 3), (0, 4, 6, 5)) * -1.0
    t2new[np.ix_(so,sO,sV,sV)] += einsum(v.oOOV, (0, 1, 2, 3), t3, (1, 4, 2, 5, 6, 3), (0, 4, 5, 6)) * -0.5
    t2new[np.ix_(so,sO,sV,sV)] += einsum(v.oOOV, (0, 1, 2, 3), t3, (1, 4, 2, 3, 5, 6), (0, 4, 6, 5)) * 0.5
    t2new[np.ix_(sO,so,sV,sV)] += einsum(v.oOOV, (0, 1, 2, 3), t3, (1, 4, 2, 5, 6, 3), (4, 0, 6, 5)) * -0.5
    t2new[np.ix_(sO,so,sV,sV)] += einsum(v.oOOV, (0, 1, 2, 3), t3, (1, 4, 2, 3, 5, 6), (4, 0, 5, 6)) * 0.5
    t2new[np.ix_(sO,so,sV,sV)] += einsum(v.oOOV, (0, 1, 2, 3), t3, (4, 1, 2, 5, 6, 3), (4, 0, 5, 6)) * -1.0
    t2new[np.ix_(sO,sO,sv,sV)] += einsum(v.OVvV, (0, 1, 2, 3), t3, (4, 5, 0, 1, 6, 3), (4, 5, 2, 6)) * -0.5
    t2new[np.ix_(sO,sO,sv,sV)] += einsum(v.OVvV, (0, 1, 2, 3), t3, (4, 5, 0, 3, 6, 1), (4, 5, 2, 6)) * 0.5
    t2new[np.ix_(sO,sO,sV,sv)] += einsum(v.OVvV, (0, 1, 2, 3), t3, (4, 5, 0, 6, 3, 1), (4, 5, 6, 2))
    t2new[np.ix_(sO,sO,sV,sv)] += einsum(v.OVvV, (0, 1, 2, 3), t3, (4, 5, 0, 1, 6, 3), (5, 4, 6, 2)) * -0.5
    t2new[np.ix_(sO,sO,sV,sv)] += einsum(v.OVvV, (0, 1, 2, 3), t3, (4, 5, 0, 3, 6, 1), (5, 4, 6, 2)) * 0.5
    t2new[np.ix_(sO,sO,sv,sV)] += einsum(v.OVvV, (0, 1, 2, 3), t3, (4, 5, 0, 6, 3, 1), (5, 4, 2, 6))
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.ov, (0, 1), t1[np.ix_(so,sv)], (2, 1), t2[np.ix_(so,so,sv,sv)], (3, 0, 4, 5), (2, 3, 5, 4)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.ov, (0, 1), t1[np.ix_(so,sv)], (2, 1), t2[np.ix_(so,so,sv,sv)], (3, 0, 4, 5), (3, 2, 4, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.ov, (0, 1), t1[np.ix_(so,sv)], (0, 2), t2[np.ix_(so,so,sv,sv)], (3, 4, 1, 5), (3, 4, 2, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(f.ov, (0, 1), t1[np.ix_(so,sv)], (0, 2), t2[np.ix_(so,so,sv,sv)], (3, 4, 5, 1), (3, 4, 5, 2)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.oovv, (4, 2, 5, 1), (0, 4, 5, 3)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.oovv, (4, 2, 5, 1), (4, 0, 3, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.oooo, (4, 0, 5, 2), (4, 5, 1, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovov, (4, 5, 2, 1), (0, 4, 3, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovov, (4, 5, 2, 1), (4, 0, 5, 3)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.vvvv, (4, 1, 5, 3), (0, 2, 4, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 2, 1), (0, 6, 4, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 2, 3, 1), (6, 0, 4, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 0, 4), (2, 6, 1, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 0, 3, 5), (6, 2, 1, 4)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 0, 3, 4), (6, 2, 1, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 0, 5), (6, 2, 1, 4))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 0, 3, 5), (2, 6, 4, 1)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 0, 3, 4), (2, 6, 5, 1))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 0, 5), (2, 6, 4, 1))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 0, 4), (6, 2, 5, 1))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 0, 3, 1), (2, 6, 4, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 0, 1), (2, 6, 4, 5)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 0, 3, 1), (6, 2, 5, 4))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ooov, (6, 3, 0, 1), (6, 2, 5, 4)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 1, 6, 5), (0, 2, 6, 4)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 1, 6, 4), (0, 2, 5, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 5, 6, 1), (0, 2, 6, 4)) * 2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 4, 6, 1), (0, 2, 6, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 1, 6, 5), (2, 0, 4, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 1, 6, 4), (2, 0, 6, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 5, 6, 1), (2, 0, 4, 6)) * 2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (3, 4, 6, 1), (2, 0, 5, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (0, 4, 6, 5), (2, 3, 1, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (0, 5, 6, 4), (2, 3, 6, 1)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (0, 1, 6, 5), (2, 3, 4, 6)) * 2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (0, 1, 6, 4), (2, 3, 6, 5)) * 2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (0, 5, 6, 1), (2, 3, 4, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(so,so,sv,sv)], (2, 3, 4, 5), v.ovvv, (0, 4, 6, 1), (2, 3, 6, 5)) * -1.0
    t2new[np.ix_(so,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.OvOV, (2, 1, 3, 4), t3, (5, 2, 3, 6, 7, 4), (0, 5, 7, 6)) * -1.0
    t2new[np.ix_(so,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.OvOV, (2, 1, 3, 4), t3, (2, 5, 3, 6, 7, 4), (0, 5, 6, 7)) * -0.5
    t2new[np.ix_(so,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.OVOv, (2, 3, 4, 1), t3, (2, 5, 4, 6, 7, 3), (0, 5, 6, 7)) * 0.5
    t2new[np.ix_(sO,so,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.OvOV, (2, 1, 3, 4), t3, (2, 5, 3, 6, 7, 4), (5, 0, 7, 6)) * -0.5
    t2new[np.ix_(sO,so,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.OVOv, (2, 3, 4, 1), t3, (5, 4, 2, 6, 7, 3), (5, 0, 6, 7)) * -1.0
    t2new[np.ix_(sO,so,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.OVOv, (2, 3, 4, 1), t3, (2, 5, 4, 6, 7, 3), (5, 0, 7, 6)) * 0.5
    t2new[np.ix_(sO,sO,sv,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOV, (0, 2, 3, 4), t3, (5, 6, 3, 2, 7, 4), (5, 6, 1, 7)) * -0.5
    t2new[np.ix_(sO,sO,sv,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOV, (0, 2, 3, 4), t3, (5, 6, 3, 4, 7, 2), (5, 6, 1, 7)) * 0.5
    t2new[np.ix_(sO,sO,sv,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOV, (0, 2, 3, 4), t3, (5, 6, 3, 7, 2, 4), (6, 5, 1, 7)) * -1.0
    t2new[np.ix_(sO,sO,sV,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOV, (0, 2, 3, 4), t3, (5, 6, 3, 2, 7, 4), (6, 5, 7, 1)) * -0.5
    t2new[np.ix_(sO,sO,sV,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOV, (0, 2, 3, 4), t3, (5, 6, 3, 4, 7, 2), (6, 5, 7, 1)) * 0.5
    t2new[np.ix_(sO,sO,sV,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOV, (0, 2, 3, 4), t3, (5, 6, 3, 7, 2, 4), (5, 6, 7, 1)) * -1.0
    t2new[np.ix_(sO,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (4, 5, 6, 7)) * 2.0
    t2new[np.ix_(sO,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (5, 4, 7, 6)) * 2.0
    t2new[np.ix_(sO,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOv, (0, 2, 3, 1), t3, (4, 5, 3, 6, 7, 2), (4, 5, 6, 7)) * -1.0
    t2new[np.ix_(sO,sO,sV,sV)] += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVOv, (0, 2, 3, 1), t3, (4, 5, 3, 6, 7, 2), (5, 4, 7, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 3, 5, 7), (0, 4, 2, 6)) * 4.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 7, 5, 6), (0, 4, 2, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 3, 5, 6), (0, 4, 2, 7)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 2, 5, 7), (0, 4, 3, 6)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 2, 5, 6), (0, 4, 3, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 3, 5, 2), (0, 4, 7, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 7, 5, 3), (0, 4, 2, 6)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 6, 5, 7), (0, 4, 2, 3)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 6, 5, 3), (0, 4, 2, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 7, 5, 2), (0, 4, 3, 6))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 3, 5, 2), (0, 4, 7, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 6, 5, 2), (0, 4, 7, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (1, 2, 5, 3), (0, 4, 7, 6))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 3, 5, 7), (0, 1, 2, 6)) * -0.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 3, 5, 6), (0, 1, 2, 7)) * 0.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 2, 5, 7), (0, 1, 6, 3)) * -1.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 2, 5, 3), (0, 1, 6, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 2, 5, 6), (0, 1, 7, 3)) * 0.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 7, 5, 3), (0, 1, 2, 6)) * 0.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 6, 5, 3), (0, 1, 2, 7)) * -1.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 7, 5, 2), (0, 1, 6, 3)) * 0.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t2[np.ix_(so,so,sv,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 6, 5, 2), (0, 1, 7, 3)) * -0.5
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), v.ooov, (6, 4, 2, 1), (0, 6, 3, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), v.ooov, (6, 2, 4, 1), (6, 0, 3, 5))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), v.ovvv, (4, 1, 6, 3), (0, 2, 5, 6)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), v.ovvv, (4, 3, 6, 1), (0, 2, 6, 5)) * -1.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (4, 1, 5, 3), (0, 2, 6, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 1, 5, 7), (0, 4, 3, 6)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 1, 5, 6), (0, 4, 3, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 7, 5, 1), (0, 4, 3, 6))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 6, 5, 1), (0, 4, 7, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 1, 5, 3), (0, 4, 7, 6))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 3, 5, 1), (0, 4, 7, 6)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 6, 5, 1), (4, 0, 3, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 1, 5, 7), (4, 0, 6, 3)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 1, 5, 6), (4, 0, 7, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 7, 5, 1), (4, 0, 6, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 1, 5, 3), (4, 0, 6, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (2, 3, 5, 1), (4, 0, 6, 7)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (0, 6, 2, 7), (4, 5, 1, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (0, 1, 2, 6), (4, 5, 3, 7)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (0, 6, 2, 1), (4, 5, 3, 7))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (0, 1, 2, 7), (4, 5, 6, 3)) * -2.0
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t2[np.ix_(so,so,sv,sv)], (4, 5, 6, 7), v.ovov, (0, 7, 2, 1), (4, 5, 6, 3))
    t2new[np.ix_(so,so,sv,sv)] += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), t1[np.ix_(so,sv)], (4, 5), t1[np.ix_(so,sv)], (6, 7), v.ovov, (4, 1, 6, 3), (0, 2, 5, 7))
    t3new = np.zeros(((naocc, naocc, naocc, navir, navir, navir)), dtype=types[float])
    t3new += einsum(f.OO, (0, 1), t3, (2, 3, 1, 4, 5, 6), (0, 3, 2, 4, 5, 6))
    t3new += einsum(f.OO, (0, 1), t3, (2, 1, 3, 4, 5, 6), (2, 0, 3, 4, 5, 6)) * -1.0
    t3new += einsum(f.OO, (0, 1), t3, (2, 3, 1, 4, 5, 6), (2, 3, 0, 4, 5, 6)) * -1.0
    t3new += einsum(f.VV, (0, 1), t3, (2, 3, 4, 5, 6, 1), (2, 3, 4, 0, 6, 5)) * -1.0
    t3new += einsum(f.VV, (0, 1), t3, (2, 3, 4, 5, 1, 6), (2, 3, 4, 5, 0, 6))
    t3new += einsum(f.VV, (0, 1), t3, (2, 3, 4, 5, 6, 1), (2, 3, 4, 5, 6, 0))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (0, 4, 5, 2, 3, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (0, 4, 5, 6, 3, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (0, 5, 4, 2, 6, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (0, 5, 4, 3, 6, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (4, 0, 5, 3, 2, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (4, 0, 5, 6, 2, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (5, 0, 4, 3, 2, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (5, 0, 4, 6, 2, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (4, 5, 0, 2, 6, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (4, 5, 0, 3, 6, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (5, 4, 0, 2, 3, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.OoOV, (4, 1, 5, 6), (5, 4, 0, 6, 3, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 3), (0, 4, 1, 2, 5, 6))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 3), (0, 4, 1, 6, 5, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 2), (0, 4, 1, 6, 5, 3))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 2), (0, 4, 1, 3, 5, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 3), (0, 1, 4, 5, 6, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 2), (0, 1, 4, 5, 3, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 3), (0, 1, 4, 2, 6, 5))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 2), (0, 1, 4, 6, 3, 5))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 3), (4, 0, 1, 5, 2, 6))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 2), (4, 0, 1, 5, 6, 3))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 3), (4, 0, 1, 6, 2, 5)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVVv, (4, 5, 6, 2), (4, 0, 1, 3, 6, 5)) * -1.0
    t3new += einsum(v.OOOO, (0, 1, 2, 3), t3, (4, 3, 1, 5, 6, 7), (0, 2, 4, 5, 6, 7)) * -1.0
    t3new += einsum(v.OOOO, (0, 1, 2, 3), t3, (1, 4, 3, 5, 6, 7), (0, 4, 2, 5, 6, 7)) * 0.5
    t3new += einsum(v.OOOO, (0, 1, 2, 3), t3, (1, 4, 3, 5, 6, 7), (0, 4, 2, 7, 6, 5)) * -0.5
    t3new += einsum(v.OOOO, (0, 1, 2, 3), t3, (4, 1, 3, 5, 6, 7), (4, 0, 2, 5, 6, 7))
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 5, 1, 6, 7, 3), (0, 5, 4, 2, 7, 6)) * -1.0
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 5, 1, 6, 3, 7), (0, 5, 4, 6, 2, 7))
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 5, 1, 6, 7, 3), (0, 5, 4, 6, 7, 2))
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 1, 5, 6, 7, 3), (4, 0, 5, 2, 7, 6))
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 1, 5, 6, 3, 7), (4, 0, 5, 6, 2, 7)) * -1.0
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 1, 5, 6, 7, 3), (4, 0, 5, 6, 7, 2)) * -1.0
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 5, 1, 6, 7, 3), (4, 5, 0, 2, 7, 6))
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 5, 1, 6, 3, 7), (4, 5, 0, 6, 2, 7)) * -1.0
    t3new += einsum(v.OOVV, (0, 1, 2, 3), t3, (4, 5, 1, 6, 7, 3), (4, 5, 0, 6, 7, 2)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (0, 4, 5, 1, 6, 7))
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (0, 5, 4, 1, 7, 6))
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (0, 4, 5, 7, 6, 1)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (0, 5, 4, 6, 7, 1)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (4, 0, 5, 6, 1, 7))
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 3, 7), (4, 0, 5, 6, 1, 7)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (4, 0, 5, 7, 1, 6)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 2, 5, 6, 3, 7), (4, 0, 5, 6, 1, 7))
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (4, 5, 0, 1, 7, 6)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (5, 4, 0, 1, 6, 7)) * -1.0
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (4, 5, 0, 6, 7, 1))
    t3new += einsum(v.OVOV, (0, 1, 2, 3), t3, (4, 5, 2, 6, 7, 3), (5, 4, 0, 7, 6, 1))
    t3new += einsum(v.VVVV, (0, 1, 2, 3), t3, (4, 5, 6, 7, 3, 1), (4, 5, 6, 0, 2, 7)) * -1.0
    t3new += einsum(v.VVVV, (0, 1, 2, 3), t3, (4, 5, 6, 1, 7, 3), (4, 5, 6, 0, 7, 2)) * 0.5
    t3new += einsum(v.VVVV, (0, 1, 2, 3), t3, (4, 5, 6, 3, 7, 1), (4, 5, 6, 0, 7, 2)) * -0.5
    t3new += einsum(v.VVVV, (0, 1, 2, 3), t3, (4, 5, 6, 7, 1, 3), (4, 5, 6, 7, 0, 2))
    t3new += einsum(f.Ov, (0, 1), t1[np.ix_(sO,sv)], (2, 1), t3, (3, 4, 0, 5, 6, 7), (2, 4, 3, 5, 6, 7))
    t3new += einsum(f.Ov, (0, 1), t1[np.ix_(sO,sv)], (2, 1), t3, (3, 0, 4, 5, 6, 7), (3, 2, 4, 5, 6, 7)) * -1.0
    t3new += einsum(f.Ov, (0, 1), t1[np.ix_(sO,sv)], (2, 1), t3, (3, 4, 0, 5, 6, 7), (3, 4, 2, 5, 6, 7)) * -1.0
    t3new += einsum(f.oV, (0, 1), t1[np.ix_(so,sV)], (0, 2), t3, (3, 4, 5, 6, 7, 1), (3, 4, 5, 2, 7, 6))
    t3new += einsum(f.oV, (0, 1), t1[np.ix_(so,sV)], (0, 2), t3, (3, 4, 5, 6, 1, 7), (3, 4, 5, 6, 2, 7)) * -1.0
    t3new += einsum(f.oV, (0, 1), t1[np.ix_(so,sV)], (0, 2), t3, (3, 4, 5, 6, 7, 1), (3, 4, 5, 6, 7, 2)) * -1.0
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 1), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 5, 3, 4, 6, 7)) * -1.0
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 1), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 5, 3, 7, 6, 4))
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 1, 4), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 5, 3, 4, 6, 7))
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 1, 4), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 5, 3, 7, 6, 4)) * -1.0
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 0, 3, 4), t2[np.ix_(sO,sO,sV,sv)], (5, 6, 7, 1), (2, 5, 6, 3, 7, 4)) * -1.0
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 0, 3, 4), t2[np.ix_(sO,sO,sv,sV)], (5, 6, 1, 7), (2, 5, 6, 3, 4, 7)) * -1.0
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 0, 3, 4), t2[np.ix_(sO,sO,sV,sv)], (5, 6, 7, 1), (2, 5, 6, 4, 7, 3))
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 0, 3, 4), t2[np.ix_(sO,sO,sv,sV)], (5, 6, 1, 7), (2, 5, 6, 7, 4, 3))
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 1), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 3, 5, 4, 7, 6)) * -1.0
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 1), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 3, 5, 6, 7, 4))
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 1, 4), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 3, 5, 6, 4, 7))
    t3new += einsum(f.ov, (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 1, 4), t2[np.ix_(sO,so,sV,sV)], (5, 0, 6, 7), (2, 3, 5, 7, 4, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (0, 2, 6, 7, 4, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (0, 2, 6, 5, 4, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (0, 6, 2, 4, 5, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (0, 6, 2, 7, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (2, 0, 6, 4, 7, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (2, 0, 6, 5, 7, 4))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (6, 0, 2, 4, 7, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (6, 0, 2, 5, 7, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (2, 6, 0, 4, 5, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (2, 6, 0, 7, 5, 4))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (6, 2, 0, 7, 4, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoVv, (6, 3, 7, 1), (6, 2, 0, 5, 4, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 3, 7, 0), (2, 6, 7, 1, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 0, 7, 3), (6, 2, 7, 1, 4, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 3, 7, 0), (6, 2, 7, 1, 4, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 0, 7, 3), (6, 7, 2, 1, 5, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 0, 7, 3), (2, 6, 7, 4, 1, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 0, 7, 3), (2, 6, 7, 5, 1, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 3, 7, 0), (6, 7, 2, 4, 1, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 3, 7, 0), (6, 7, 2, 5, 1, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 3, 7, 0), (2, 6, 7, 4, 5, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 0, 7, 3), (6, 2, 7, 5, 4, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 3, 7, 0), (6, 2, 7, 5, 4, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OoOo, (6, 0, 7, 3), (6, 7, 2, 4, 5, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 5), (2, 3, 6, 1, 7, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 4), (2, 3, 6, 1, 5, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 5), (6, 2, 3, 1, 4, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 4), (6, 2, 3, 1, 7, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 5), (2, 6, 3, 4, 1, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 5), (2, 6, 3, 7, 1, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 4), (2, 6, 3, 7, 1, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 4), (2, 6, 3, 5, 1, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 5), (2, 3, 6, 4, 7, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 4), (2, 3, 6, 7, 5, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 5), (6, 2, 3, 7, 4, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OoVv, (6, 0, 7, 4), (6, 2, 3, 5, 7, 1))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (0, 2, 6, 7, 4, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (0, 2, 6, 5, 4, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (0, 6, 2, 4, 7, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (0, 6, 2, 5, 7, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (2, 0, 6, 7, 5, 4))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (2, 0, 6, 4, 5, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (6, 0, 2, 7, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (6, 0, 2, 4, 5, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (2, 6, 0, 4, 7, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (2, 6, 0, 5, 7, 4))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (6, 2, 0, 7, 4, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 3, 1), (6, 2, 0, 5, 4, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 5), (2, 6, 3, 1, 7, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 4), (2, 6, 3, 1, 7, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 4), (2, 3, 6, 1, 5, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 5), (6, 2, 3, 1, 4, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 5), (2, 3, 6, 7, 1, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 5), (2, 3, 6, 4, 1, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 4), (6, 2, 3, 7, 1, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 4), (6, 2, 3, 5, 1, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 5), (2, 6, 3, 4, 7, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 4), (2, 6, 3, 5, 7, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 4), (2, 3, 6, 7, 5, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.OVov, (6, 7, 0, 5), (6, 2, 3, 7, 4, 1)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.VvVv, (6, 1, 7, 5), (0, 2, 3, 6, 4, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.VvVv, (6, 1, 7, 4), (0, 2, 3, 6, 7, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.VvVv, (6, 5, 7, 1), (0, 2, 3, 6, 4, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.VvVv, (6, 4, 7, 1), (0, 2, 3, 5, 6, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.VvVv, (6, 5, 7, 1), (2, 0, 3, 6, 7, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.VvVv, (6, 4, 7, 1), (2, 0, 3, 6, 7, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.VvVv, (6, 1, 7, 5), (2, 0, 3, 4, 6, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.VvVv, (6, 1, 7, 4), (2, 0, 3, 5, 6, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.VvVv, (6, 1, 7, 5), (2, 3, 0, 6, 7, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.VvVv, (6, 1, 7, 4), (2, 3, 0, 6, 5, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), v.VvVv, (6, 4, 7, 1), (2, 3, 0, 6, 5, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), v.VvVv, (6, 5, 7, 1), (2, 3, 0, 4, 6, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (5, 3, 4, 6, 7, 8), (0, 2, 5, 6, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (3, 5, 4, 6, 7, 8), (0, 5, 2, 6, 7, 8)) * -0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (3, 5, 4, 6, 7, 8), (0, 5, 2, 8, 7, 6)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (5, 4, 3, 6, 7, 8), (2, 0, 5, 6, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (5, 4, 3, 6, 7, 8), (5, 0, 2, 6, 7, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (3, 5, 4, 6, 7, 8), (2, 5, 0, 8, 7, 6)) * -0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (3, 5, 4, 6, 7, 8), (2, 5, 0, 6, 7, 8)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OOOv, (2, 3, 4, 1), t3, (5, 3, 4, 6, 7, 8), (5, 2, 0, 6, 7, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (2, 5, 6, 1, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (2, 6, 5, 1, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 6, 3, 7, 8, 4), (2, 6, 5, 1, 8, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 3, 6, 7, 8, 4), (5, 2, 6, 1, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (5, 6, 2, 1, 8, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (6, 5, 2, 1, 7, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 6, 3, 7, 8, 4), (5, 6, 2, 1, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 6, 3, 7, 4, 8), (2, 6, 5, 7, 1, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (5, 2, 6, 7, 1, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (5, 2, 6, 8, 1, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 4, 8), (5, 2, 6, 7, 1, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 3, 6, 7, 4, 8), (5, 2, 6, 7, 1, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 3, 6, 7, 4, 8), (5, 2, 6, 7, 1, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 6, 3, 7, 4, 8), (5, 6, 2, 7, 1, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (2, 5, 6, 8, 7, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (2, 6, 5, 7, 8, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 6, 3, 7, 8, 4), (2, 6, 5, 7, 8, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 3, 6, 7, 8, 4), (5, 2, 6, 7, 8, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (5, 6, 2, 7, 8, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OoOV, (2, 0, 3, 4), t3, (5, 6, 3, 7, 8, 4), (6, 5, 2, 8, 7, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.OOoV, (2, 3, 0, 4), t3, (5, 6, 3, 7, 8, 4), (5, 6, 2, 7, 8, 1))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.OoOv, (2, 0, 3, 1), t3, (4, 5, 3, 6, 7, 8), (2, 5, 4, 6, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.OOov, (2, 3, 0, 1), t3, (4, 5, 3, 6, 7, 8), (2, 5, 4, 6, 7, 8)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.OoOv, (2, 0, 3, 1), t3, (4, 3, 5, 6, 7, 8), (4, 2, 5, 6, 7, 8))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.OOov, (2, 3, 0, 1), t3, (4, 3, 5, 6, 7, 8), (4, 2, 5, 6, 7, 8)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.OoOv, (2, 0, 3, 1), t3, (4, 5, 3, 6, 7, 8), (4, 5, 2, 6, 7, 8))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.OOov, (2, 3, 0, 1), t3, (4, 5, 3, 6, 7, 8), (4, 5, 2, 6, 7, 8)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 6, 2, 7, 8, 4), (0, 6, 5, 3, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 6, 2, 7, 4, 8), (0, 6, 5, 7, 3, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 6, 2, 7, 8, 4), (0, 6, 5, 7, 8, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (0, 5, 6, 4, 7, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (0, 6, 5, 4, 8, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (0, 5, 6, 8, 7, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (0, 6, 5, 7, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 2, 6, 7, 8, 4), (5, 0, 6, 3, 8, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 2, 6, 7, 4, 8), (5, 0, 6, 7, 3, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 2, 6, 7, 8, 4), (5, 0, 6, 7, 8, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (5, 0, 6, 8, 4, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 3, 8), (5, 0, 6, 7, 4, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (5, 0, 6, 7, 4, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 2, 6, 7, 3, 8), (5, 0, 6, 7, 4, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 6, 2, 7, 8, 4), (5, 6, 0, 3, 8, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 6, 2, 7, 4, 8), (5, 6, 0, 7, 3, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OvVV, (2, 1, 3, 4), t3, (5, 6, 2, 7, 8, 4), (5, 6, 0, 7, 8, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (5, 6, 0, 4, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (6, 5, 0, 4, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (5, 6, 0, 7, 8, 4))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), v.OVVv, (2, 3, 4, 1), t3, (5, 6, 2, 7, 8, 3), (6, 5, 0, 8, 7, 4))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 8, 4, 2), (5, 6, 7, 1, 3, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 2, 8, 4), (5, 6, 7, 1, 8, 3)) * -0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 4, 8, 2), (5, 6, 7, 1, 8, 3)) * 0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 8, 2, 4), (5, 6, 7, 3, 1, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 8, 2, 4), (5, 6, 7, 8, 1, 3)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 4, 8, 2), (5, 6, 7, 3, 8, 1)) * -0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 2, 8, 4), (5, 6, 7, 3, 8, 1)) * 0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), v.oVVV, (0, 2, 3, 4), t3, (5, 6, 7, 8, 4, 2), (5, 6, 7, 8, 3, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovVV, (0, 1, 2, 3), t3, (4, 5, 6, 7, 8, 3), (4, 5, 6, 2, 8, 7)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovVV, (0, 1, 2, 3), t3, (4, 5, 6, 7, 3, 8), (4, 5, 6, 7, 2, 8)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.ovVV, (0, 1, 2, 3), t3, (4, 5, 6, 7, 8, 3), (4, 5, 6, 7, 8, 2)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVVv, (0, 2, 3, 1), t3, (4, 5, 6, 7, 8, 2), (4, 5, 6, 3, 8, 7))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVVv, (0, 2, 3, 1), t3, (4, 5, 6, 7, 2, 8), (4, 5, 6, 7, 3, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), v.oVVv, (0, 2, 3, 1), t3, (4, 5, 6, 7, 8, 2), (4, 5, 6, 7, 8, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 7), (0, 4, 8, 2, 6, 3)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 3), (0, 4, 8, 2, 6, 7))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 6), (0, 4, 8, 2, 7, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 7), (0, 4, 8, 3, 6, 2)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 3), (0, 4, 8, 7, 6, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 6), (0, 4, 8, 3, 7, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 2), (0, 4, 8, 3, 6, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 2), (0, 4, 8, 7, 6, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 7), (0, 4, 8, 2, 6, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (0, 4, 8, 2, 6, 7)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 6), (0, 4, 8, 2, 3, 7))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 7), (0, 4, 8, 3, 6, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (0, 4, 8, 7, 6, 2)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 6), (0, 4, 8, 7, 3, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 2), (0, 4, 8, 3, 6, 7))
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 2), (0, 4, 8, 7, 6, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 7), (0, 8, 4, 2, 3, 6)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 3), (0, 8, 4, 2, 7, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 6), (0, 8, 4, 2, 3, 7))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 7), (0, 8, 4, 6, 3, 2)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 3), (0, 8, 4, 6, 7, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 6), (0, 8, 4, 7, 3, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 2), (0, 8, 4, 6, 3, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 2), (0, 8, 4, 7, 3, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 7), (0, 8, 4, 2, 3, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (0, 8, 4, 2, 7, 6)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 6), (0, 8, 4, 2, 7, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 7), (0, 8, 4, 6, 3, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (0, 8, 4, 6, 7, 2)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 6), (0, 8, 4, 3, 7, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 2), (0, 8, 4, 6, 7, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 2), (0, 8, 4, 3, 7, 6))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 3), (0, 8, 1, 2, 7, 6))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 3), (0, 8, 1, 6, 7, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 2), (0, 8, 1, 6, 7, 3))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 2), (0, 8, 1, 3, 7, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 3), (0, 1, 8, 2, 7, 6))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 3), (0, 1, 8, 6, 7, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 2), (0, 1, 8, 6, 3, 7)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 2), (0, 1, 8, 7, 3, 6)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 2), (0, 1, 8, 6, 3, 7)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 2), (0, 1, 8, 7, 3, 6)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (8, 0, 4, 6, 2, 7))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (8, 0, 4, 7, 2, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 2), (8, 0, 4, 6, 3, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 2), (8, 0, 4, 7, 3, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 7), (8, 0, 4, 6, 2, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (8, 0, 4, 6, 2, 7))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 7), (8, 0, 4, 3, 2, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 3), (8, 0, 4, 7, 2, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 6), (8, 0, 4, 3, 2, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 1, 6), (8, 0, 4, 7, 2, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 7), (8, 0, 4, 6, 2, 3)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 3), (8, 0, 4, 6, 2, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 7), (8, 0, 4, 3, 2, 6)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 3), (8, 0, 4, 7, 2, 6))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 6), (8, 0, 4, 3, 2, 7))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 6), (8, 0, 4, 7, 2, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 2), (8, 0, 4, 6, 7, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 1, 5, 2), (8, 0, 4, 3, 7, 6))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 3), (8, 0, 1, 6, 2, 7)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 3), (8, 0, 1, 7, 2, 6)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 2), (8, 0, 1, 6, 7, 3))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 4, 5, 2), (8, 0, 1, 3, 7, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 3), (8, 0, 1, 6, 2, 7)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(so,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 4, 3), (8, 0, 1, 7, 2, 6)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 7), (0, 4, 1, 2, 6, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 6), (0, 4, 1, 2, 8, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 7), (0, 4, 1, 8, 6, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 6), (0, 4, 1, 7, 8, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 7), (0, 4, 1, 8, 6, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 3), (0, 4, 1, 8, 6, 7))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 7), (0, 4, 1, 3, 6, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 3), (0, 4, 1, 7, 6, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 6), (0, 4, 1, 3, 8, 7))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 6), (0, 4, 1, 7, 8, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 3), (0, 4, 1, 2, 6, 8)) * 2.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 3), (0, 4, 1, 2, 7, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 3), (0, 4, 1, 8, 6, 2)) * -2.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 3), (0, 4, 1, 8, 7, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 2), (0, 4, 1, 8, 6, 3)) * 2.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 2), (0, 4, 1, 8, 6, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 2), (0, 4, 1, 3, 6, 8)) * -2.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 2), (0, 4, 1, 7, 6, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 2), (0, 4, 1, 8, 7, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 2), (0, 4, 1, 3, 7, 8))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 3, 8, 7), (0, 4, 5, 2, 6, 8)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sv)], (4, 5, 6, 7), v.ovVv, (1, 7, 8, 6), (0, 4, 5, 2, 8, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 3, 8, 6), (0, 4, 5, 2, 8, 7)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 3, 8, 7), (0, 4, 5, 8, 6, 2)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sv)], (4, 5, 6, 7), v.ovVv, (1, 7, 8, 6), (0, 4, 5, 3, 8, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 3, 8, 6), (0, 4, 5, 7, 8, 2)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 2, 8, 7), (0, 4, 5, 8, 6, 3))
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 2, 8, 7), (0, 4, 5, 3, 6, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 2, 8, 6), (0, 4, 5, 3, 8, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 2, 8, 6), (0, 4, 5, 7, 8, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 7, 8, 3), (0, 4, 5, 2, 6, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 6, 8, 3), (0, 4, 5, 2, 8, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sv)], (4, 5, 6, 7), v.ovVv, (1, 6, 8, 7), (0, 4, 5, 2, 3, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 7, 8, 3), (0, 4, 5, 8, 6, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sv)], (4, 5, 6, 7), v.ovVv, (1, 6, 8, 7), (0, 4, 5, 8, 3, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 6, 8, 3), (0, 4, 5, 7, 8, 2))
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 7, 8, 2), (0, 4, 5, 8, 6, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (1, 7, 8, 2), (0, 4, 5, 3, 6, 8))
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 6, 8, 2), (0, 4, 5, 8, 3, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (1, 6, 8, 2), (0, 4, 5, 7, 3, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 7), (0, 1, 4, 2, 8, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 6), (0, 1, 4, 2, 7, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 7), (0, 1, 4, 6, 8, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 6), (0, 1, 4, 8, 7, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 3), (0, 1, 4, 6, 8, 7))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 7), (0, 1, 4, 6, 3, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 7), (0, 1, 4, 8, 3, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 3), (0, 1, 4, 7, 8, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 6), (0, 1, 4, 8, 3, 7))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 2, 8, 6), (0, 1, 4, 7, 3, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 3), (0, 1, 4, 2, 8, 6)) * 2.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 3), (0, 1, 4, 2, 8, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 3), (0, 1, 4, 6, 8, 2)) * -2.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 3), (0, 1, 4, 7, 8, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 2), (0, 1, 4, 6, 3, 8)) * -2.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 2), (0, 1, 4, 6, 7, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sv)], (4, 5, 6, 7), v.ovVv, (5, 7, 8, 2), (0, 1, 4, 8, 3, 6)) * 2.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 2), (0, 1, 4, 8, 7, 6)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 2), (0, 1, 4, 8, 3, 7)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), t2[np.ix_(sO,so,sv,sV)], (4, 5, 6, 7), v.ovVv, (5, 6, 8, 2), (0, 1, 4, 7, 3, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (4, 7, 5, 8, 9, 6), (0, 1, 7, 2, 8, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (4, 7, 5, 8, 9, 6), (0, 1, 7, 9, 8, 2)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (7, 4, 5, 8, 9, 6), (0, 1, 7, 8, 3, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (7, 4, 5, 8, 6, 9), (0, 1, 7, 8, 3, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (7, 4, 5, 8, 9, 6), (0, 1, 7, 9, 3, 8)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (7, 5, 4, 8, 6, 9), (0, 1, 7, 8, 3, 9))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), v.OvOv, (4, 2, 5, 3), t3, (6, 5, 4, 7, 8, 9), (0, 1, 6, 7, 8, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (7, 6, 4, 8, 9, 5), (0, 1, 7, 2, 9, 8)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (4, 7, 6, 8, 9, 5), (0, 1, 7, 2, 8, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (7, 6, 4, 8, 9, 5), (0, 1, 7, 8, 9, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (4, 7, 6, 8, 9, 5), (0, 1, 7, 9, 8, 2)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (7, 4, 6, 8, 9, 5), (0, 1, 7, 8, 3, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (7, 4, 6, 8, 5, 9), (0, 1, 7, 8, 3, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (7, 4, 6, 8, 9, 5), (0, 1, 7, 9, 3, 8)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (7, 4, 5, 8, 9, 6), (0, 7, 1, 2, 8, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (4, 7, 5, 8, 9, 6), (0, 7, 1, 2, 9, 8)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (7, 4, 5, 8, 9, 6), (0, 7, 1, 9, 8, 2))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (4, 7, 5, 8, 9, 6), (0, 7, 1, 8, 9, 2)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (7, 4, 5, 8, 9, 6), (0, 7, 1, 3, 8, 9))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (7, 4, 5, 8, 9, 6), (0, 7, 1, 9, 8, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (4, 7, 5, 8, 9, 6), (0, 7, 1, 3, 9, 8)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (4, 7, 5, 8, 9, 6), (0, 7, 1, 8, 9, 3)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), v.OvOv, (4, 2, 5, 3), t3, (4, 6, 5, 7, 8, 9), (0, 6, 1, 7, 8, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (4, 7, 6, 8, 9, 5), (0, 7, 1, 2, 9, 8)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (4, 7, 6, 8, 9, 5), (0, 7, 1, 8, 9, 2)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (4, 7, 6, 8, 9, 5), (0, 7, 1, 3, 9, 8)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (4, 7, 6, 8, 9, 5), (0, 7, 1, 8, 9, 3)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), v.OvOv, (4, 3, 5, 2), t3, (4, 6, 5, 7, 8, 9), (0, 6, 1, 7, 8, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (0, 7, 8, 2, 3, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (0, 7, 8, 2, 3, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (0, 7, 8, 2, 9, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 6, 7, 2, 8, 9)) * 2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (0, 8, 7, 2, 9, 3)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (0, 8, 7, 2, 9, 3)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 7, 6, 2, 9, 8)) * 2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (0, 7, 8, 3, 9, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (0, 7, 8, 9, 3, 2)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (0, 7, 8, 9, 3, 2)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 6, 7, 9, 8, 2)) * -2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (0, 8, 7, 3, 9, 2)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (0, 8, 7, 3, 9, 2)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 7, 6, 8, 9, 2)) * -2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 6, 7, 3, 8, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 6, 7, 9, 8, 3)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 7, 6, 3, 9, 8)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 7, 6, 8, 9, 3)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 3, 4, 2), t3, (5, 6, 4, 7, 8, 9), (0, 6, 5, 7, 8, 9))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (0, 6, 7, 2, 8, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (0, 8, 7, 2, 3, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (0, 7, 6, 2, 9, 8)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (0, 6, 7, 9, 8, 2)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (0, 8, 7, 9, 3, 2))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (0, 7, 6, 8, 9, 2)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 7, 5, 8, 9, 4), (0, 7, 6, 3, 9, 8)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 7, 5, 8, 4, 9), (0, 7, 6, 8, 3, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 7, 5, 8, 9, 4), (0, 7, 6, 8, 9, 3)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 3, 4, 2), t3, (5, 6, 4, 7, 8, 9), (0, 6, 5, 7, 8, 9))
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 2, 4, 3), t3, (5, 6, 4, 7, 8, 9), (0, 6, 5, 7, 8, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (7, 4, 5, 8, 9, 6), (7, 0, 1, 8, 2, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (7, 4, 5, 8, 6, 9), (7, 0, 1, 8, 2, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (7, 4, 5, 8, 9, 6), (7, 0, 1, 9, 2, 8)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OvOV, (4, 3, 5, 6), t3, (7, 5, 4, 8, 6, 9), (7, 0, 1, 8, 2, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (4, 7, 5, 8, 9, 6), (7, 0, 1, 3, 8, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OvOV, (4, 2, 5, 6), t3, (4, 7, 5, 8, 9, 6), (7, 0, 1, 9, 8, 3)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sv)], (0, 1, 2, 3), v.OvOv, (4, 3, 5, 2), t3, (6, 5, 4, 7, 8, 9), (6, 0, 1, 7, 8, 9))
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (7, 4, 6, 8, 9, 5), (7, 0, 1, 8, 2, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (7, 4, 6, 8, 5, 9), (7, 0, 1, 8, 2, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sV,sv)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 3), t3, (7, 4, 6, 8, 9, 5), (7, 0, 1, 9, 2, 8)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (7, 6, 4, 8, 9, 5), (7, 0, 1, 3, 9, 8))
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (7, 6, 4, 8, 9, 5), (7, 0, 1, 8, 9, 3)) * -1.0
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (4, 7, 6, 8, 9, 5), (7, 0, 1, 3, 8, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,sO,sv,sV)], (0, 1, 2, 3), v.OVOv, (4, 5, 6, 2), t3, (4, 7, 6, 8, 9, 5), (7, 0, 1, 9, 8, 3)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 9, 4, 6), (7, 0, 8, 3, 2, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 9, 6, 4), (7, 0, 8, 3, 2, 9)) * 2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 4, 9, 6), (7, 0, 8, 3, 2, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 6, 9, 4), (7, 0, 8, 3, 2, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 9, 4, 6), (7, 0, 8, 9, 2, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 9, 6, 4), (7, 0, 8, 9, 2, 3)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 4, 9, 6), (7, 0, 8, 9, 2, 3)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 5, 8, 6, 9, 4), (7, 0, 8, 9, 2, 3)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 0, 7, 8, 2, 9)) * 2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 5, 9), (6, 0, 7, 8, 2, 9)) * -2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 0, 7, 9, 2, 8)) * -2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 4, 7, 8, 5, 9), (6, 0, 7, 8, 2, 9)) * 2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 0, 7, 8, 3, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 5, 9), (6, 0, 7, 8, 3, 9)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 0, 7, 9, 3, 8)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 4, 7, 8, 5, 9), (6, 0, 7, 8, 3, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 3, 4, 2), t3, (5, 4, 6, 7, 8, 9), (5, 0, 6, 7, 8, 9)) * -2.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (6, 0, 7, 8, 2, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 4, 9), (6, 0, 7, 8, 2, 9)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (6, 0, 7, 9, 2, 8)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 5, 7, 8, 4, 9), (6, 0, 7, 8, 2, 9)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 5, 7, 8, 9, 4), (6, 0, 7, 3, 9, 8)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 5, 7, 8, 4, 9), (6, 0, 7, 8, 3, 9)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 5, 7, 8, 9, 4), (6, 0, 7, 8, 9, 3)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 2, 4, 3), t3, (5, 4, 6, 7, 8, 9), (5, 0, 6, 7, 8, 9))
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (7, 8, 0, 2, 9, 3)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (7, 8, 0, 2, 9, 3)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 7, 0, 2, 9, 8)) * -2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (8, 7, 0, 2, 3, 9)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (8, 7, 0, 2, 3, 9)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (8, 7, 0, 2, 9, 3))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (7, 6, 0, 2, 8, 9)) * -2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (7, 8, 0, 3, 9, 2)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (7, 8, 0, 3, 9, 2)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 7, 0, 8, 9, 2)) * 2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (8, 7, 0, 3, 9, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 4, 9, 6), (8, 7, 0, 9, 3, 2)) * -0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 6, 9, 4), (8, 7, 0, 9, 3, 2)) * 0.5
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.ovOV, (1, 3, 4, 5), t3, (6, 7, 4, 8, 9, 5), (7, 6, 0, 9, 8, 2)) * 2.00000000000002
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 7, 0, 3, 9, 8)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 7, 0, 8, 9, 3)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (7, 6, 0, 3, 8, 9)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.ovOV, (1, 2, 4, 5), t3, (6, 7, 4, 8, 9, 5), (7, 6, 0, 9, 8, 3)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 3, 4, 2), t3, (5, 6, 4, 7, 8, 9), (5, 6, 0, 7, 8, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (7, 8, 0, 2, 3, 9))
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (6, 7, 0, 2, 9, 8)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (7, 6, 0, 2, 8, 9)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sV)], (0, 1, 2, 3), v.oVOV, (1, 4, 5, 6), t3, (7, 8, 5, 9, 4, 6), (7, 8, 0, 9, 3, 2)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (6, 7, 0, 8, 9, 2)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sV,sv)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 3), t3, (6, 7, 5, 8, 9, 4), (7, 6, 0, 9, 8, 2)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 7, 5, 8, 9, 4), (6, 7, 0, 3, 9, 8)) * -1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 7, 5, 8, 4, 9), (6, 7, 0, 8, 3, 9)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sV)], (0, 1, 2, 3), v.oVOv, (1, 4, 5, 2), t3, (6, 7, 5, 8, 9, 4), (6, 7, 0, 8, 9, 3)) * 1.00000000000001
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 3, 4, 2), t3, (5, 6, 4, 7, 8, 9), (5, 6, 0, 7, 8, 9)) * -1.0
    t3new += einsum(t2[np.ix_(sO,so,sv,sv)], (0, 1, 2, 3), v.ovOv, (1, 2, 4, 3), t3, (5, 6, 4, 7, 8, 9), (5, 6, 0, 7, 8, 9))
    t3new += einsum(t2[np.ix_(so,so,sV,sV)], (0, 1, 2, 3), v.oVoV, (0, 4, 1, 5), t3, (6, 7, 8, 9, 5, 4), (6, 7, 8, 2, 3, 9)) * -1.0
    t3new += einsum(t2[np.ix_(so,so,sV,sV)], (0, 1, 2, 3), v.oVoV, (0, 4, 1, 5), t3, (6, 7, 8, 4, 9, 5), (6, 7, 8, 2, 9, 3)) * 0.25
    t3new += einsum(t2[np.ix_(so,so,sV,sV)], (0, 1, 2, 3), v.oVoV, (0, 4, 1, 5), t3, (6, 7, 8, 5, 9, 4), (6, 7, 8, 2, 9, 3)) * -0.25
    t3new += einsum(t2[np.ix_(so,so,sV,sv)], (0, 1, 2, 3), v.ovoV, (0, 3, 1, 4), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 2, 9, 8)) * -0.5
    t3new += einsum(t2[np.ix_(so,so,sV,sv)], (0, 1, 2, 3), v.ovoV, (0, 3, 1, 4), t3, (5, 6, 7, 8, 4, 9), (5, 6, 7, 8, 2, 9)) * 0.5
    t3new += einsum(t2[np.ix_(so,so,sV,sV)], (0, 1, 2, 3), v.oVoV, (0, 4, 1, 5), t3, (6, 7, 8, 4, 9, 5), (6, 7, 8, 3, 9, 2)) * -0.25
    t3new += einsum(t2[np.ix_(so,so,sV,sV)], (0, 1, 2, 3), v.oVoV, (0, 4, 1, 5), t3, (6, 7, 8, 5, 9, 4), (6, 7, 8, 3, 9, 2)) * 0.25
    t3new += einsum(t2[np.ix_(so,so,sV,sV)], (0, 1, 2, 3), v.oVoV, (0, 4, 1, 5), t3, (6, 7, 8, 9, 5, 4), (6, 7, 8, 9, 3, 2))
    t3new += einsum(t2[np.ix_(so,so,sV,sv)], (0, 1, 2, 3), v.ovoV, (0, 3, 1, 4), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 8, 9, 2)) * 0.5
    t3new += einsum(t2[np.ix_(so,so,sv,sV)], (0, 1, 2, 3), v.ovoV, (0, 2, 1, 4), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 3, 9, 8)) * 0.5
    t3new += einsum(t2[np.ix_(so,so,sv,sV)], (0, 1, 2, 3), v.ovoV, (0, 2, 1, 4), t3, (5, 6, 7, 8, 4, 9), (5, 6, 7, 8, 3, 9)) * -1.5
    t3new += einsum(t2[np.ix_(so,so,sv,sV)], (0, 1, 2, 3), v.ovoV, (0, 2, 1, 4), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 8, 9, 3)) * -0.5
    t3new += einsum(t2[np.ix_(so,so,sV,sv)], (0, 1, 2, 3), v.oVov, (0, 4, 1, 3), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 2, 9, 8)) * 1.5
    t3new += einsum(t2[np.ix_(so,so,sV,sv)], (0, 1, 2, 3), v.oVov, (0, 4, 1, 3), t3, (5, 6, 7, 8, 4, 9), (5, 6, 7, 8, 2, 9)) * -0.5
    t3new += einsum(t2[np.ix_(so,so,sV,sv)], (0, 1, 2, 3), v.oVov, (0, 4, 1, 3), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 8, 9, 2)) * -1.5
    t3new += einsum(t2[np.ix_(so,so,sv,sV)], (0, 1, 2, 3), v.oVov, (0, 4, 1, 2), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 3, 9, 8)) * -0.5
    t3new += einsum(t2[np.ix_(so,so,sv,sV)], (0, 1, 2, 3), v.oVov, (0, 4, 1, 2), t3, (5, 6, 7, 8, 4, 9), (5, 6, 7, 8, 3, 9)) * 0.5
    t3new += einsum(t2[np.ix_(so,so,sv,sV)], (0, 1, 2, 3), v.oVov, (0, 4, 1, 2), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 8, 9, 3)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (0, 4, 8, 3, 6, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (0, 4, 8, 3, 6, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (0, 8, 4, 3, 7, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (0, 8, 4, 6, 3, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (0, 8, 4, 7, 3, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (0, 4, 8, 7, 6, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (0, 4, 8, 7, 6, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (0, 8, 4, 6, 7, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (4, 0, 8, 3, 7, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (8, 0, 4, 3, 7, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (4, 0, 8, 6, 3, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (4, 0, 8, 7, 3, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (8, 0, 4, 6, 3, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (8, 0, 4, 7, 3, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (4, 0, 8, 6, 7, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (8, 0, 4, 6, 7, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (4, 8, 0, 3, 7, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (8, 4, 0, 3, 6, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (8, 4, 0, 3, 6, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (4, 8, 0, 6, 3, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (4, 8, 0, 7, 3, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (4, 8, 0, 6, 7, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 5, 1), (8, 4, 0, 7, 6, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.Ooov, (8, 5, 2, 1), (8, 4, 0, 7, 6, 3))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 2, 0, 7), (4, 8, 5, 1, 3, 6)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 0, 6), (4, 8, 5, 1, 3, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 0, 2, 7), (4, 5, 8, 1, 3, 6)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 0, 2, 6), (8, 4, 5, 1, 3, 7))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 0, 2, 6), (4, 5, 8, 1, 7, 3)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 0, 6), (4, 5, 8, 1, 7, 3))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 0, 2, 7), (8, 4, 5, 1, 6, 3))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 2, 0, 7), (8, 4, 5, 1, 6, 3)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 0, 2, 7), (4, 8, 5, 6, 1, 3))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 0, 2, 6), (4, 8, 5, 7, 1, 3)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.Ooov, (8, 2, 0, 7), (4, 5, 8, 6, 1, 3))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.Ooov, (8, 2, 0, 6), (8, 4, 5, 7, 1, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 1, 8, 3), (0, 2, 4, 6, 8, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 1, 8, 3), (0, 2, 4, 7, 8, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 1), (0, 2, 4, 6, 7, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 1), (0, 2, 4, 8, 7, 6)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 1, 8, 3), (0, 4, 2, 8, 6, 7))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 1, 8, 3), (0, 4, 2, 7, 6, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 1), (0, 4, 2, 8, 6, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 1), (0, 4, 2, 7, 6, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 7), (0, 4, 5, 3, 6, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 6), (0, 4, 5, 3, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 7, 8, 1), (0, 4, 5, 3, 6, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 6, 8, 1), (0, 4, 5, 8, 3, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 6, 8, 1), (0, 4, 5, 7, 3, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 7), (0, 4, 5, 8, 6, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 6), (0, 4, 5, 7, 8, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 7, 8, 1), (0, 4, 5, 8, 6, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 1, 8, 3), (4, 0, 2, 6, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 1, 8, 3), (4, 0, 2, 8, 7, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 1), (4, 0, 2, 6, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t2[np.ix_(sO,so,sV,sV)], (4, 5, 6, 7), v.ovVv, (5, 3, 8, 1), (4, 0, 2, 7, 8, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 7, 8, 1), (4, 0, 5, 3, 8, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 6, 8, 1), (4, 0, 5, 3, 8, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 7), (4, 0, 5, 6, 3, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 7), (4, 0, 5, 8, 3, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 6), (4, 0, 5, 8, 3, 7)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 6), (4, 0, 5, 7, 3, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 7, 8, 1), (4, 0, 5, 6, 8, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 6, 8, 1), (4, 0, 5, 7, 8, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 7), (4, 5, 0, 3, 8, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 6), (4, 5, 0, 3, 7, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 6, 8, 1), (4, 5, 0, 3, 7, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 7, 8, 1), (4, 5, 0, 6, 3, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 7, 8, 1), (4, 5, 0, 8, 3, 6))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sV,sv)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 7), (4, 5, 0, 6, 8, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 1, 8, 6), (4, 5, 0, 8, 7, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t2[np.ix_(sO,sO,sv,sV)], (4, 5, 6, 7), v.ovVv, (2, 6, 8, 1), (4, 5, 0, 8, 7, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), v.OvOv, (4, 1, 5, 3), t3, (6, 5, 4, 7, 8, 9), (0, 2, 6, 7, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), v.OvOv, (4, 1, 5, 3), t3, (4, 6, 5, 7, 8, 9), (0, 6, 2, 7, 8, 9)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), v.OvOv, (4, 3, 5, 1), t3, (4, 6, 5, 7, 8, 9), (0, 6, 2, 7, 8, 9)) * -0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 6, 7, 3, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 7, 6, 3, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 7, 5, 8, 9, 4), (0, 7, 6, 3, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 7, 5, 8, 4, 9), (0, 7, 6, 8, 3, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 6, 7, 9, 8, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (0, 7, 6, 8, 9, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 7, 5, 8, 9, 4), (0, 7, 6, 8, 9, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovOv, (2, 1, 4, 3), t3, (5, 6, 4, 7, 8, 9), (0, 6, 5, 7, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovOv, (2, 3, 4, 1), t3, (5, 6, 4, 7, 8, 9), (0, 6, 5, 7, 8, 9)) * 2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), v.OvOv, (4, 3, 5, 1), t3, (6, 5, 4, 7, 8, 9), (6, 0, 2, 7, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 5, 7, 8, 9, 4), (6, 0, 7, 3, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 0, 7, 8, 3, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 0, 7, 9, 3, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 5, 9), (6, 0, 7, 8, 3, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 4, 7, 8, 5, 9), (6, 0, 7, 8, 3, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 5, 7, 8, 4, 9), (6, 0, 7, 8, 3, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 5, 7, 8, 9, 4), (6, 0, 7, 8, 9, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovOv, (2, 1, 4, 3), t3, (5, 4, 6, 7, 8, 9), (5, 0, 6, 7, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovOv, (2, 3, 4, 1), t3, (5, 4, 6, 7, 8, 9), (5, 0, 6, 7, 8, 9)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 7, 0, 3, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (7, 6, 0, 3, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 7, 5, 8, 9, 4), (6, 7, 0, 3, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 7, 5, 8, 4, 9), (6, 7, 0, 8, 3, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (6, 7, 0, 8, 9, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovOV, (2, 1, 4, 5), t3, (6, 7, 4, 8, 9, 5), (7, 6, 0, 9, 8, 3)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVOv, (2, 4, 5, 1), t3, (6, 7, 5, 8, 9, 4), (6, 7, 0, 8, 9, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovOv, (2, 1, 4, 3), t3, (5, 6, 4, 7, 8, 9), (5, 6, 0, 7, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sv)], (2, 3), v.ovOv, (2, 3, 4, 1), t3, (5, 6, 4, 7, 8, 9), (5, 6, 0, 7, 8, 9)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVoV, (0, 4, 2, 5), t3, (6, 7, 8, 9, 5, 4), (6, 7, 8, 1, 3, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVoV, (0, 4, 2, 5), t3, (6, 7, 8, 5, 9, 4), (6, 7, 8, 1, 9, 3)) * -0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVoV, (0, 4, 2, 5), t3, (6, 7, 8, 4, 9, 5), (6, 7, 8, 1, 9, 3)) * 0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVoV, (0, 4, 2, 5), t3, (6, 7, 8, 9, 4, 5), (6, 7, 8, 9, 1, 3))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovoV, (0, 1, 2, 4), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 3, 9, 8)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVov, (0, 4, 2, 1), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 3, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovoV, (0, 1, 2, 4), t3, (5, 6, 7, 8, 4, 9), (5, 6, 7, 8, 3, 9)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVov, (0, 4, 2, 1), t3, (5, 6, 7, 8, 4, 9), (5, 6, 7, 8, 3, 9))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.ovoV, (0, 1, 2, 4), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 8, 9, 3)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), v.oVov, (0, 4, 2, 1), t3, (5, 6, 7, 8, 9, 4), (5, 6, 7, 8, 9, 3))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 5, 7, 1), (0, 2, 6, 8, 4, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 5, 7, 1), (0, 2, 6, 9, 4, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 4, 7, 1), (0, 2, 6, 8, 5, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 4, 7, 1), (0, 2, 6, 9, 5, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (7, 1, 3, 9), (0, 2, 6, 8, 4, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (7, 1, 3, 5), (0, 2, 6, 8, 4, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (7, 1, 3, 9), (0, 2, 6, 5, 4, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (7, 1, 3, 5), (0, 2, 6, 9, 4, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (7, 1, 3, 8), (0, 2, 6, 5, 4, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (7, 1, 3, 8), (0, 2, 6, 9, 4, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (7, 9, 3, 1), (0, 2, 6, 8, 4, 5)) * 2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (7, 5, 3, 1), (0, 2, 6, 8, 4, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (7, 9, 3, 1), (0, 2, 6, 5, 4, 8)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (7, 5, 3, 1), (0, 2, 6, 9, 4, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (7, 8, 3, 1), (0, 2, 6, 5, 4, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (7, 8, 3, 1), (0, 2, 6, 9, 4, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (7, 4, 3, 1), (0, 2, 6, 8, 9, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (7, 4, 3, 1), (0, 2, 6, 5, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 5), (0, 2, 3, 8, 4, 9)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 5), (0, 2, 3, 9, 4, 8)) * -0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 4), (0, 2, 3, 8, 9, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 4), (0, 2, 3, 5, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 5, 7, 1), (0, 2, 3, 8, 4, 9)) * -0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 5, 7, 1), (0, 2, 3, 9, 4, 8)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 9), (2, 0, 6, 4, 5, 8)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 5), (2, 0, 6, 4, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 8), (2, 0, 6, 4, 5, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 9), (2, 0, 6, 8, 5, 4)) * 2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 5), (2, 0, 6, 8, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 8), (2, 0, 6, 9, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 4), (2, 0, 6, 8, 5, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 4), (2, 0, 6, 9, 5, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 9, 7, 1), (2, 0, 6, 4, 5, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 5, 7, 1), (2, 0, 6, 4, 9, 8)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 8, 7, 1), (2, 0, 6, 4, 9, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 9, 7, 1), (2, 0, 6, 8, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 5, 7, 1), (2, 0, 6, 8, 9, 4)) * 2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 8, 7, 1), (2, 0, 6, 5, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 4, 7, 1), (2, 0, 6, 8, 9, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 4, 7, 1), (2, 0, 6, 5, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 5, 7, 1), (2, 0, 3, 4, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 5, 7, 1), (2, 0, 3, 8, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 4, 7, 1), (2, 0, 3, 8, 9, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 4, 7, 1), (2, 0, 3, 5, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 9), (2, 6, 0, 4, 8, 5)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 5), (2, 6, 0, 4, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 8), (2, 6, 0, 4, 9, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 9), (2, 6, 0, 5, 8, 4)) * 2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 5), (2, 6, 0, 9, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 8), (2, 6, 0, 5, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 4), (2, 6, 0, 5, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 1, 7, 4), (2, 6, 0, 9, 8, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 9, 7, 1), (2, 6, 0, 4, 8, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 5, 7, 1), (2, 6, 0, 4, 8, 9)) * -2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 8, 7, 1), (2, 6, 0, 4, 5, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (3, 9, 7, 1), (2, 6, 0, 5, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 5, 7, 1), (2, 6, 0, 9, 8, 4)) * 2.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (3, 8, 7, 1), (2, 6, 0, 9, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 4, 7, 1), (2, 6, 0, 5, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (3, 4, 7, 1), (2, 6, 0, 9, 8, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 5), (2, 3, 0, 4, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 5), (2, 3, 0, 8, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 4), (2, 3, 0, 8, 5, 9)) * -0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 1, 7, 4), (2, 3, 0, 9, 5, 8)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 4, 7, 1), (2, 3, 0, 8, 5, 9)) * 0.5
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(so,so,sV,sV)], (6, 7, 8, 9), v.ovov, (6, 4, 7, 1), (2, 3, 0, 9, 5, 8)) * -0.5
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 9), (2, 6, 3, 1, 8, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 8), (2, 6, 3, 1, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 9), (2, 6, 3, 1, 8, 5)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 5), (2, 6, 3, 1, 8, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 8), (2, 6, 3, 1, 9, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 5), (2, 6, 3, 1, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 4), (2, 6, 3, 1, 8, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 4), (2, 6, 3, 1, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 5, 3, 9), (2, 6, 7, 1, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 8), (2, 6, 7, 1, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 4, 3, 9), (2, 6, 7, 1, 8, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 3, 8), (2, 6, 7, 1, 5, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 5), (2, 6, 7, 1, 8, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 4), (2, 6, 7, 1, 8, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 9), (2, 3, 6, 1, 5, 8)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 5), (2, 3, 6, 1, 9, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 8), (2, 3, 6, 1, 5, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 5), (2, 3, 6, 1, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 4), (2, 3, 6, 1, 5, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 4), (2, 3, 6, 1, 5, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 5), (2, 6, 3, 4, 1, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 5), (2, 6, 3, 9, 1, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 4), (2, 6, 3, 5, 1, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 4), (2, 6, 3, 9, 1, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 3, 8), (2, 6, 7, 4, 1, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 3, 8), (2, 6, 7, 9, 1, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sv)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 9), (2, 6, 7, 4, 1, 5))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 5), (2, 6, 7, 4, 1, 9)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sv)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 9), (2, 6, 7, 5, 1, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 5), (2, 6, 7, 9, 1, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 4), (2, 6, 7, 5, 1, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 4), (2, 6, 7, 9, 1, 5)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 9), (2, 3, 6, 4, 1, 8)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 8), (2, 3, 6, 4, 1, 9))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 9), (2, 3, 6, 8, 1, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 8), (2, 3, 6, 9, 1, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 5), (2, 3, 6, 4, 1, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 5), (2, 3, 6, 8, 1, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 4), (2, 3, 6, 8, 1, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 4), (2, 3, 6, 9, 1, 8))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 9), (2, 6, 3, 4, 8, 1)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 8), (2, 6, 3, 4, 9, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 9), (2, 6, 3, 5, 8, 1)) * 2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 5), (2, 6, 3, 9, 8, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 8), (2, 6, 3, 5, 9, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 5), (2, 6, 3, 4, 8, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 4), (2, 6, 3, 5, 8, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 4), (2, 6, 3, 9, 8, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 5, 3, 9), (2, 6, 7, 4, 8, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 8), (2, 6, 7, 4, 5, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 4, 3, 9), (2, 6, 7, 5, 8, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 3, 8), (2, 6, 7, 9, 5, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 5), (2, 6, 7, 4, 8, 1)) * -2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,so,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 4), (2, 6, 7, 5, 8, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 9), (2, 3, 6, 8, 5, 1)) * 2.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 5), (2, 3, 6, 8, 9, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 8), (2, 3, 6, 9, 5, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 5), (2, 3, 6, 4, 9, 1))
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 7, 4), (2, 3, 6, 8, 5, 1)) * -1.0
    t3new += einsum(t1[np.ix_(so,sV)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 7, 4), (2, 3, 6, 9, 5, 1))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 5), (2, 6, 3, 4, 8, 9)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 5), (2, 6, 3, 9, 8, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 4), (2, 6, 3, 5, 8, 9)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 4), (2, 6, 3, 9, 8, 5)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 1), (2, 6, 3, 4, 8, 9))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 1), (2, 6, 3, 9, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 1), (2, 6, 3, 5, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 1), (2, 6, 3, 9, 8, 5))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 1, 3, 9), (2, 6, 7, 4, 8, 5)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 3, 8), (2, 6, 7, 4, 5, 9)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 1, 3, 9), (2, 6, 7, 5, 8, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 3, 8), (2, 6, 7, 9, 5, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 1), (2, 6, 7, 4, 8, 5))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 1), (2, 6, 7, 4, 5, 9))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (0, 9, 3, 1), (2, 6, 7, 5, 8, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,so,sV,sV)], (2, 3, 4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (0, 8, 3, 1), (2, 6, 7, 9, 5, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 5), (2, 3, 6, 4, 9, 8)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 5), (2, 3, 6, 8, 9, 4)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 4), (2, 3, 6, 8, 5, 9)) * 2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 1, 7, 4), (2, 3, 6, 9, 5, 8)) * -2.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 1), (2, 3, 6, 4, 9, 8))
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sV,sv)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 5, 7, 1), (2, 3, 6, 8, 9, 4)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 1), (2, 3, 6, 8, 5, 9)) * -1.0
    t3new += einsum(t1[np.ix_(so,sv)], (0, 1), t2[np.ix_(sO,sO,sv,sV)], (2, 3, 4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (0, 4, 7, 1), (2, 3, 6, 9, 5, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 1, 7, 3), (0, 2, 6, 5, 9, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 3, 7, 1), (0, 2, 6, 8, 5, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 3, 7, 1), (0, 2, 6, 9, 5, 8))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 1, 7, 3), (0, 2, 6, 8, 9, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 1, 7, 3), (0, 6, 2, 5, 8, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 3, 7, 1), (0, 6, 2, 5, 8, 9)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 1, 7, 3), (0, 6, 2, 9, 8, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 3, 7, 1), (0, 6, 2, 9, 8, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (2, 1, 4, 8), (0, 6, 7, 3, 5, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (2, 1, 4, 9), (0, 6, 7, 3, 8, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (2, 9, 4, 1), (0, 6, 7, 3, 8, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (2, 8, 4, 1), (0, 6, 7, 9, 3, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 3, 7, 1), (6, 0, 2, 5, 9, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 1, 7, 3), (6, 0, 2, 8, 5, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 1, 7, 3), (6, 0, 2, 9, 5, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(sO,sv)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,so,sV,sV)], (6, 7, 8, 9), v.ovov, (4, 3, 7, 1), (6, 0, 2, 8, 9, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (2, 9, 4, 1), (6, 0, 7, 3, 5, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (2, 8, 4, 1), (6, 0, 7, 3, 5, 9))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (2, 1, 4, 9), (6, 0, 7, 8, 3, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (2, 1, 4, 8), (6, 0, 7, 9, 3, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (2, 1, 4, 9), (6, 7, 0, 3, 5, 8)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (2, 1, 4, 8), (6, 7, 0, 3, 9, 5)) * -1.0
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sv,sV)], (6, 7, 8, 9), v.ovov, (2, 8, 4, 1), (6, 7, 0, 3, 9, 5))
    t3new += einsum(t1[np.ix_(sO,sv)], (0, 1), t1[np.ix_(so,sV)], (2, 3), t1[np.ix_(so,sV)], (4, 5), t2[np.ix_(sO,sO,sV,sv)], (6, 7, 8, 9), v.ovov, (2, 9, 4, 1), (6, 7, 0, 8, 3, 5))

    return {"t1new": t1new, "t2new": t2new, "t3new": t3new}

